<?php

require_once("lib-mbapi/include/modules/module.php");
require_once("lib-mbapi/include/curl/curl.php");
require_once("lib-billing/include/config/ssl.php");

require_once("UI/Form.php");

use Billing\Object\Session;
use Billing\Object\Todo;

abstract class SSLModule extends MB_Module
{
	/**
	 * The constant to represent a successful SSL module execution.
	 */
	const STATUS_SUCCESS = "success";

	/**
	 * The constant to represent a failed SSL module execution.
	 */
	const STATUS_FAILURE = "failure";

	/**
	 * The constant to represent a non-finished operation.
	 */
	const ACTION_STATUS_PENDING = "pending";

	/**
	 * The constant to represent a completed operation.
	 */
	const ACTION_STATUS_COMPLETED = "completed";

	/**
	 * The constant to represent a errored operation.
	 */
	const ACTION_STATUS_ERROR = "error";

	/**
	 * The constant to represent a successful connection to a physical SSL remote API during execution.
	 */
	const STATUS_CONNECTION_SUCCESS = "connection_success";

	/**
	 * The constant to represent a failed connection to a physical panel during execution.
	 */
	const STATUS_CONNECTION_FAILURE = "connection_failure";

	/**
	 * The constant to represent a failed requirement during parameter validation.
	 */
	const STATUS_PARAM_FAILURE = "param_failure";

	/**
	 * The constant to identify a SSL's add capability.
	 */
	const CAPABILITY_ADD = "capability_add";

	/**
	 * The constant to identify a SSL's delete capability (uninstall from panel only).
	 */
	const CAPABILITY_DELETE = "capability_delete";

	/**
	 * The constant to identify a SSL's get issued cert capability.
	 */
	const CAPABILITY_GETISSUEDCERT = "capability_getissuedcert";

	/**
	 * The constant to identify a SSL's get CSR capability (used for getting the CSR from remote).
	 */
	const CAPABILITY_GETCSR = "capability_getcsr";

	/**
	 * The constant to identify a SSL's get issued cert capability.
	 */
	const CAPABILITY_RENEWCERT = "capability_renewcert";

	const CAPABILITY_PRODUCT_PARAMS = "capability_product_params";
	const CAPABILITY_USER_PARAMS = "capability_user_params";

	const CRYPTOR_SECRET_KEY = "39845f7h29845cna8a87vb5slkfb6qc4bbv462vitdonv";

	/**
	 * Config Input Type (CIT)
	 */
	const CIT_TEXT		= "t";
	const CIT_TEXT_AREA	= "a";
	const CIT_RADIO		= "r";
	const CIT_SELECT	= "s";
	const CIT_BOOL		= "b";
	const CIT_PASSWORD	= "p";

	public $connectionInfo;
    public $attributes;

    abstract public function getCertificates();
    abstract public function getPeriod($certificate);
    abstract public function getCertificateAttributeName();
    abstract public function getPeriodAttributeName();

	public function __construct(array $input)
	{
		$this->input = $input;
		$this->prepareModuleEnv();

		// include module specific translations
		if (file_exists($transDir = dirname(__FILE__) . "/translations/" . get_class($this))) {
			BillingLocale::addTranslationsDir($transDir);
		}

		if (0 == count($this->input))
		{
			$this->status = SSLModule::STATUS_PARAM_FAILURE;
			return;
		}
		if (isset($this->input["subCommand"]))
		{
			$method = $this->input["subCommand"];
			$this->$method();
		}
	}

	/**
	* Returns the module display for the module. This includes version number and display name.
	* the columns in config_params. This array will be looped by SetConfigGroups.
	*
	* @return    An array of arrays containing all the display info for the module.
	*/
	public function getModuleInfo()
	{
		/**
		 * This method must return an array of the module info with the next data:
		 *
		 *  'type' => 'ssl'
		 *     // Always 'ssl'.
		 *
		 *  'status' => 'TRANS_STABLE'
		 *     // Can be 'TRANS_STABLE', 'TRANS_TESTING'.
		 *
		 *  'version' => '1.0.0'
		 *     // A version of this module
		 *
		 *  'displayName' => 'DummySSL'.
		 *     // Display name used in modules list.
		 *
		 *  'author' => 'Parallels'
		 *     // Author of this module which was implemented this module.
		 *
		 */
		$this->dieMandatoryMethod(__METHOD__);
	}

	/**
	* Returns the specified module info.
	*
	* @return A scalar value.
	*/
	public function getModuleInfoData($data_name)
	{
		$module_info = $this->getModuleInfo();
		$data_value = NULL;

		if (isset($module_info[$data_name]))
			$data_value = $module_info[$data_name];

		return $data_value;
	}

	public static function getPluginInstance($plugin_id, array $params)
	{
		$params["pluginID"] = $plugin_id;
		require_once("ModuleTool.php");
		return ModuleTool::getModuleInstance("sslmodule", Factory()->SSLPlugin($plugin_id)->ssl_plugin_module, $params);
	}

	/**
	 * @return  An array of capabilities of this Panel object.
	 */
	public function getCapabilities()
	{
		return $this->capabilities;
	}

	/**
	 * @return  TRUE if this instantiation is valid, and FALSE if otherwise.
	 */
	public function isValid()
	{
		return $this->valid;
	}

	public function addConfigParams(UIForm $ui_form)
	{
		$this->drawConfigParams($ui_form, $this->config);
	}

	public function collectConfigParams(UIForm $ui_form)
	{
		return $this->cryptParams(
			$this->getCollectedConfigParams($ui_form, $this->config),
			$this->cryptedConfigParams()
		);
	}

	public function addProductParams(UIForm $ui_form)
	{
		if ($this->isCapable(self::CAPABILITY_PRODUCT_PARAMS))
			$this->drawProductParams($ui_form, $this->config, $this->product);
	}

	public function collectProductParams(UIForm $ui_form)
	{
		if (! $this->isCapable(self::CAPABILITY_PRODUCT_PARAMS))
			return array();

		return $this->cryptParams(
			$this->getCollectedProductParams($ui_form, $this->config, $this->product),
			$this->cryptedProductParams()
		);
	}

	public function addUserParams(UIForm $ui_form)
	{
		if ($this->isCapable(self::CAPABILITY_USER_PARAMS))
			$this->drawUserParams($ui_form, $this->config, $this->ssl_attributes);
	}

	public function getUserParams()
	{
		return ($this->isCapable(self::CAPABILITY_USER_PARAMS)) ? $this->getUserParamsDescriptor($this->config, $this->ssl_attributes) : NULL ;
	}

	public function collectUserParams(UIForm $ui_form)
	{
		if (! $this->isCapable(self::CAPABILITY_USER_PARAMS))
			return array();

		return $this->cryptParams( $this->getCollectedUserParamsFromUIForm($ui_form, $this->config, $this->ssl_attributes), $this->cryptedUserParams());
	}

	public function collectUserParamsFromArray(array $collected_values)
	{
		if (! $this->isCapable(self::CAPABILITY_USER_PARAMS))
			return array();

		return $this->cryptParams( $this->getCollectedUserParamsFromArray($this->config, $this->ssl_attributes, $collected_values), $this->cryptedUserParams());
	}



	/**
	 * This method can be rewritten by children and should returns the list of
	 * config params, which must be crypted before storing into DB.
	 */
	public function cryptedConfigParams()
	{
		return array();
	}

	/**
	 * This method can be rewritten by children and should returns the list of
	 * product attributes names, which must be crypted before storing into DB.
	 */
	public function cryptedProductParams()
	{
		return array();
	}

	/**
	 * This method can be rewritten by children and should returns the list of
	 * package attributes names, which must be crypted before storing into DB.
	 */
	public function cryptedUserParams()
	{
		return array(
			"PVT", "SSLCertificate",
		);
	}

	/**
	 * This method will return the XML string with curl connection info if that data exists.  Time
	 * items will be in seconds with millisecond resolution and the size items will be in units of bytes
	 *
	 * @return  string  The XML string containing the cURL connection results.
	 */
	public function getReturnXML()
	{

		//Build the XML
		$connectionXML = $this->getCurlInfo();
		$result  = "<results>\n";
		$result .= "<result>\n";
		$result .= "<response>\n";

		if (is_array($this->arrayData))
			$result .= toXML($this->arrayData);

		$result .= "</response>\n";
		$result .= $connectionXML;
		$result .= "</result>\n";
		$result .= "</results>\n";

		return $this->getXMLHeader().$result.$this->getXMLFooter();

	}

	public function isDone()
	{
		return $this->done;
	}

	/**
	 *  This method will return if the panel has a specific capability
	 *  @param int Ex. PANEL_CLIENTLOGIN
	 *  @return int Is this panel capabable of requested feature.
	 */
	public function isCapable($capability)
	{
		return in_array($capability, $this->capabilities);
	}


	abstract public function getParamGroupsMap();

	final public function add()
	{
		try
		{
			$resp = $this->pluginAdd($this->config, $this->ssl_attributes);
			$this->checkResponse($resp, "add");
		}
		catch (ProductException $e)
		{
			$this->pushErrorToAdminsToDo($error = $e->getMessage());
			return array(
				"execution_status" => self::STATUS_FAILURE,
				"error" => $error,
			);
		}
        catch (\Billing\Module\Exception $e)
        {
            $this->pushErrorToAdminsToDo($error = $e->getMessage());
            return array(
                "execution_status" => self::STATUS_FAILURE,
                "error" => $error,
            );
        }
		if (isset($resp["error"]))
			$this->pushErrorToAdminsToDo($resp["error"]);

		$this->done = self::ACTION_STATUS_PENDING != $resp["status"];
		if (isset($resp["data"]))
			$this->saveAttributes($resp["data"]);

		return array(
			"execution_status" => self::STATUS_SUCCESS,
			"action_status" => $resp["status"],
			"action_status_reason" => $resp["status_reason"],
			"error" => $resp["error"],
			"result" => $resp["data"],
		);
	}

	final public function renew()
	{
		try
		{
			$resp = $this->pluginRenew($this->config, $this->ssl_attributes);
			$this->checkResponse($resp, "renew");
		}
		catch (ProductException $e)
		{
			$this->pushErrorToAdminsToDo($error = $e->getMessage());
			return array(
				"execution_status" => self::STATUS_FAILURE,
				"error" => $error,
			);
		}
		if (isset($resp["error"]))
			$this->pushErrorToAdminsToDo($resp["error"]);

		$this->done = self::ACTION_STATUS_PENDING != $resp["status"];
		if (isset($resp["data"]))
			$this->saveAttributes($resp["data"]);

		return array(
			"execution_status" => self::STATUS_SUCCESS,
			"action_status" => $resp["status"],
			"action_status_reason" => $resp["status_reason"],
			"error" => $resp["error"],
			"result" => $resp["data"],
		);
	}

	final public function delete()
	{
		try
		{
			$resp = $this->pluginDelete($this->config, $this->ssl_attributes);
			$this->checkResponse($resp, "delete");
		}
		catch (ProductException $e)
		{
			$this->pushErrorToAdminsToDo($error = $e->getMessage());
			return array(
				"execution_status" => self::STATUS_FAILURE,
				"error" => $error,
			);
		}
		if (isset($resp["error"]))
			$this->pushErrorToAdminsToDo($resp["error"]);

		$this->done = self::ACTION_STATUS_PENDING != $resp["status"];
		if (isset($resp["data"]))
			$this->saveAttributes($resp["data"]);

		return array(
			"execution_status" => self::STATUS_SUCCESS,
			"action_status" => $resp["status"],
			"action_status_reason" => $resp["status_reason"],
			"error" => $resp["error"],
			"result" => $resp["data"],
		);
	}

	final public function getCSR()
	{
		try
		{
			$resp = $this->pluginGetCSR($this->config, $this->ssl_attributes);
			$this->checkResponse($resp, "getCSR");
		}
		catch (ProductException $e)
		{
			$this->pushErrorToAdminsToDo($error = $e->getMessage());
			return array(
				"execution_status" => self::STATUS_FAILURE,
				"error" => $error,
			);
		}
		if (isset($resp["error"]))
			$this->pushErrorToAdminsToDo($resp["error"]);

		$this->done = self::ACTION_STATUS_PENDING != $resp["status"];
		if (isset($resp["data"]))
			$this->saveAttributes($resp["data"]);

		return array(
			"execution_status" => self::STATUS_SUCCESS,
			"action_status" => $resp["status"],
			"action_status_reason" => $resp["status_reason"],
			"error" => $resp["error"],
			"result" => $resp["data"],
		);
	}

	final public function getIssuedCert()
	{
		try
		{
			$resp = $this->pluginGetIssuedCert($this->config, $this->ssl_attributes);
			$this->checkResponse($resp, "getIssuedCert");
		}
		catch (ProductException $e)
		{
			$this->pushErrorToAdminsToDo($error = $e->getMessage());
			return array(
				"execution_status" => self::STATUS_FAILURE,
				"error" => $error,
			);
		}
		if (isset($resp["error"]))
			$this->pushErrorToAdminsToDo($resp["error"]);

		$this->done = self::ACTION_STATUS_PENDING != $resp["status"];
		if (isset($resp["data"]))
			$this->saveAttributes($resp["data"]);

		return array(
			"execution_status" => self::STATUS_SUCCESS,
			"action_status" => $resp["status"],
			"action_status_reason" => $resp["status_reason"],
			"error" => $resp["error"],
			"result" => $resp["data"],
		);
	}

	final public function checkAttributes($action)
	{
		return $this->pluginCheckAttributes($this->config, $this->ssl_attributes, $action);
	}

	final public function saveAttributes(array $attributes)
	{
		$package = Factory()->Subscription($this->packageID);
		foreach ($this->cryptParams($attributes, $this->cryptedUserParams()) as $name => $value)
		{
			$package->PackageAttributes[$name]->value = $value;
			$this->ssl_attributes[$name] = isset($attributes[$name]) ? $attributes[$name] : $value;
		}
		Service()->SubscriptionManager->update($package);
	}

	final public function getDomainOpts()
	{
		$package = Factory()->Subscription($this->packageID);
		return is_null($package->PrimaryDomain)
			? NULL
			: array("SubDomain" => $this->ssl_attributes["SubDomain"], "SLD" => $package->PrimaryDomain->domain_sld, "TLD" => $package->PrimaryDomain->domain_tld);
	}

	final public function getDomainName(array $domain_opts = NULL)
	{
		if (! is_null($domain_opts))
		{
			$domain_opts["SubDomain"] .= ("" == $domain_opts["SubDomain"]) ? "" : ".";
			return "{$domain_opts['SubDomain']}{$domain_opts['SLD']}.{$domain_opts['TLD']}";
		}
		if (isset($this->ssl_attributes["commonName"]))
			return $this->ssl_attributes["commonName"];
		if (!isset($this->ssl_attributes["CSR"]))
			return NULL;

		$dn = $this->getCSRSubject();
		return $dn["CN"];
	}

	final public function getCSRSubject()
	{
		return openssl_csr_get_subject($this->ssl_attributes["CSR"]);
	}

	final public function reloadAttributes()
	{
		$this->ssl_attributes = NULL;
		$this->prepareAttributes();
	}


	/********************
	 * Plugin's actions *
	 ********************/

	/**
	 * This method needs to be overridden by children.
	 *
	 * @return  result of function calling as assoc array.
	 */
	abstract protected function pluginAdd(array $config, array $attributes);

	/**
	 * This method needs to be overridden by children.
	 *
	 * @return  result of function calling as assoc array.
	 */
	abstract protected function pluginRenew(array $config, array $attributes);

	/**
	 * This method needs to be overridden by children.
	 *
	 * @return  result of function calling as assoc array.
	 */
	abstract protected function pluginDelete(array $config, array $attributes);

	/**
	 * This method needs to be overridden by children.
	 *
	 * @return  result of function calling as assoc array.
	 */
	abstract protected function pluginGetIssuedCert(array $config, array $attributes);

	/**
	 * This method needs to be overridden by children.
	 *
	 * @return  result of function calling as assoc array.
	 */
	abstract protected function pluginGetCSR(array $config, array $attributes);

	/**
	 * This method can be overridden by children to return the list of invalidated attrs.
	 *
	 * @return assoc array	an assoc array with the list of attributes and warnings passed as an item.
	 */
	protected function pluginCheckAttributes(array $config, array $attributes, $action)
	{
		// TODO: Check all needed data before remote calling. If any invalid data will be found,
		//		 this method must handle all needed parameters of SSL module to perform error to outside.
		return NULL; // But can be returned an array of errored attributes
					 // as: array( 'attrName' => 'attrWarnigMsg', ... )
	}

	/**
	 * This method needs to be overriden by children and must returns
	 * SSL module configuration set with default parameters.
	 */
	abstract protected function drawConfigParams(UIForm $ui_form, array $config);

	abstract protected function getCollectedConfigParams(UIForm $ui_form, array $config);

	abstract protected function drawProductParams(UIForm $ui_form, array $config, array $product);

	abstract protected function getCollectedProductParams(UIForm $ui_form, array $config, array $product);

	abstract protected function drawUserParams(UIForm $ui_form, array $config, array $attributes);

	abstract protected function getCollectedUserParamsFromArray(array $config, array $attributes, array $collected_values);

	abstract protected function getCollectedUserParamsFromUIForm(UIForm $ui_form, array $config, array $attributes);

	abstract protected function getUserParamsDescriptor(array $config, array $attributes);

	public static function webServerTypes($attributes)
    {
        return array();
    }

	protected function getCurlObject($url = "", $fields = "", $cli = 0, $config_identity = NULL)
	{
		if (is_null($config_identity))
			$config_identity = "SSL_".get_class($this);

		return new Curl($url, $fields, $cli, $config_identity);
	}

	/**
	 * This method will return the XML string with curl connection info if that data exists.  Time
	 * items will be in seconds with millisecond resolution and the size items will be in units of bytes
	 *
	 * @return  string  The XML string containing the cURL connection results.
	 */
	protected function getCurlInfo()
	{
		$connectionXML = "";

		if (! is_null($this->connectionInfo) && is_array($this->connectionInfo))
		{
			//Build the connection info XML
			foreach ($this->connectionInfo as $key => $value)
				$connectionXML .= "<${key}>${value}</${key}>\n";

			$connectionXML = "<curlConnectionData>\n${connectionXML}</curlConnectionData>\n";
		}

		return $connectionXML;
	}

    /**
	 * This check for curl errors and set the proper panel settings if errors are found
	 *
	 */
	protected function getCurlErrors($curl)
	{
		$curl_errors = array();

		if (0 < $curl->getNumErrors())
    	{
			$curl_errors = array();
			foreach ($curl->errors as $value)
				$curl_errors[] = array("cURL Error:", $value);

			$this->errors = array_merge($this->errors, $curl_errors);
			$this->hasExecutedSuccessfully = false;
			$this->arrayData = $curl->resultArray;
		}

		return count($curl_errors) ? $curl_errors : NULL;
	}

	/**
	 * This method returns all countries with country code and localized country name.
	 */
	protected function getCountries()
	{
		if (! is_null($this->countries))
			return $this->countries;

		$this->countries = array();
		foreach (Factory()->CountryList() as $country)
			$this->countries[$country->countries_iso_2] = blmsg("TRANS_ISO_3166_".$country->countries_iso_2);

		asort($this->countries);
		return $this->countries;
	}

	protected function getUSStates()
	{
		if (! is_null($this->us_states))
			return $this->us_states;

		$this->us_states = blmsg_array(Service()->CountryManager->getStates("US"));
		asort($this->us_states);
		return $this->us_states;
	}

	protected function getConfig()
	{
		return $this->config;
	}

	final protected function debug($entry)
	{
		dbg($entry, 0, 0, get_class($this));
	}

	final protected function isStandAloneCert()
	{
		return 0 >= Factory()->Subscription($this->packageID)->parent_package_id;
	}

	final protected function generateCSR($domain = NULL)
	{
		$client = Factory()->Subscription($this->packageID)->Customer;

		if (is_null($domain)) {
			$domain = $this->getDomainName();
		}
		$clientCompany = ("" != $client->PrimaryContact->contact_company) ? $client->PrimaryContact->contact_company : "-" ;
		$dn = array(
			"commonName"			=> $domain,
			"countryName"			=> $client->Country->countries_iso_2,
			"stateOrProvinceName"	=> $client->PrimaryContact->contact_state,
			"localityName"			=> $client->PrimaryContact->contact_city,
			"organizationName"		=> $clientCompany,
			"emailAddress"			=> $client->PrimaryContact->contact_email,
		);

		$r_pvt = openssl_pkey_new(array(
			"private_key_bits" => isset($this->ssl_attributes["PrivateKeyBits"]) ? $this->ssl_attributes["PrivateKeyBits"] : 2048,
			"private_key_type" => OPENSSL_KEYTYPE_RSA
		));
		$r_csr = openssl_csr_new($dn, $r_pvt);

		openssl_csr_export($r_csr, $csr);
		openssl_pkey_export($r_pvt, $pvt);

		return array(
			"csr" => $csr,
			"pvt" => $pvt,
		);
	}

	public function pushErrorToAdminsToDo($errors)
	{
		$errors = trim($errors);

		$todo = Factory()->Todo();
		$plugin = Factory()->SSLPlugin($this->pluginID);
        $todo->user_id = (null !== $plugin->vendor_id) ? Factory()->Reseller($plugin->vendor_id)->Admin->id : Service()->AdminManager->getMasterAdmin()->id;
        $todo->user_type = Session::USER_TYPE_ADMIN;
        $todo->vendor_id = $plugin->vendor_id;
		$todo->todo_title = blmsg("TRANS_SSL_MODULE_ERROR__REMOTE_ERROR__TODO_TITLE", array("SSL_PLUGIN_ID" => $plugin->id, "SSL_PLUGIN_NAME" => $plugin->ssl_plugin_name));
		$todo->todo_desc = ! is_null($this->packageID)
			? blmsg("TRANS_SSL_MODULE_ERROR__REMOTE_ERROR_FOR_PACKAGE__TODO_DESC", array("PACKAGE_ID" => $this->packageID, "SSL_PLUGIN_ID" => $plugin->id, "SSL_PLUGIN_NAME" => $plugin->ssl_plugin_name, "ERRORS" => $errors))
			: blmsg("TRANS_SSL_MODULE_ERROR__REMOTE_ERROR_FOR_PRODUCT__TODO_DESC", array("PRODUCT_ID" => $this->productID, "SSL_PLUGIN_ID" => $plugin->id, "SSL_PLUGIN_NAME" => $plugin->ssl_plugin_name, "ERRORS" => $errors));
		$todo->todo_is_public = 1;
		$todo->todo_priority = 3;
		$todo->todo_status_id = Todo::STATUS_OPEN;
		$todo->todo_date_created = $todo->todo_date_due = time();
		$todo->todo_link_name = "id";
		$todo->todo_link_action_name = "EditSSLPluginConfig";
		$todo->todo_link_file = "modules.php";
		$todo->todo_link_pk = $this->pluginID;

        $todo->update();
	}

	private function dieMandatoryMethod($method)
	{
		throw new Exception("[class ".get_class($this)."] The method \"${method}\" should be overwritten by child class!");
	}

	private function cryptParams(array $params, array $to_be_crypted)
	{
		foreach ($params as $name => $value)
		{
			if (is_null($value)) {
				$params[$name] = $value;
				continue;
			}

			if (in_array($name, $to_be_crypted) && false !== ($value = $this->getCryptor()->encrypt($value)))
				$params[$name] = $value;
		}
		return $params;
	}

	private function checkResponse($response, $action)
	{
		if (! is_array($response))
			throw new Exception("Response must be array");

		if (! isset($response["status"]))
			throw new Exception("Response must contains the \"status\" item");

		if (self::ACTION_STATUS_ERROR == $response["status"] && ! isset($response["error"]))
			throw new Exception("Response must contains the \"error\" item when action is completed with errors");

		switch ($action)
		{
			case "add":
			case "getIssuedCert":
			case "renew":
				if (self::ACTION_STATUS_COMPLETED == $response["status"]
					&& (! isset($response["data"])
						|| ! isset($response["data"]["CSR"])
						|| ! isset($response["data"]["SSLCertificate"]))
				) {
					$this->debug("Wrong response returned:");
					$this->debug($response);
					throw new Exception("Response must contains all the following items: \"CSR\", \"SSLCertificate\"");
				}
				break;
		}
	}

	private function prepareModuleEnv()
	{
		$this->pluginID = $this->input["pluginID"];
        if (isset($this->input["productID"])) {
            $this->productID = $this->input["productID"];
        }
        if (isset($this->input["packageID"])) {
            $this->packageID = $this->input["packageID"];
        }

		$this->hasExecuted = false;
		$this->hasExecutedSuccessfully = false;
		$this->done = false;
		$this->status = SSLModule::STATUS_FAILURE;
		$this->connectionInfo = NULL;
		$this->capabilities = array();
		$this->timestamp = time();

		$this->prepareConfig();
		$this->prepareProduct();
		$this->prepareAttributes();
	}

	private function prepareConfig()
	{
		if (! is_null($this->config))
			return $this->config;

		if (0 >= (int)$this->pluginID)
		{
			$this->config = array();
			return;
		}
		try
		{
			$this->config = DataObject::getMapOf(
				Factory()->SSLPluginParams($this->pluginID),
				"ssl_plugin_param_name",
				"ssl_plugin_param_value"
			);

			$crypted_params = $this->cryptedConfigParams();
			foreach ($this->config as $name => $value)
			{
				if (in_array($name, $crypted_params) && false !== ($value = $this->getCryptor()->decrypt($value)))
					$this->config[$name] = $value;
			}
		}
		catch (ProductException $e)
		{
			$this->valid = false;
			$this->addError(
				blmsg("TRANS_SSL_MODULE_ERROR"),
				"Cannot load config for ".get_class($this)."[{$this->pluginID}]. Error: "
					.$e->getMessage()
			);
		}
		return NULL;
	}

	private function prepareProduct()
	{
		if (! is_null($this->product))
			return $this->product;

		if (0 >= (int)$this->productID)
		{
			$this->product = array();
			return;
		}
		try
		{
			$this->product = DataObject::getMapOf(
				Factory()->ProductAttributes(array($this->productID, 0)),
				"product_attribute_name",
				"product_attribute_value"
			);
		}
		catch (ProductException $e)
		{
			$this->valid = false;
			$this->addError(
				blmsg("TRANS_SSL_MODULE_ERROR"),
				"Cannot load Product's config for ".get_class($this)."[{$this->pluginID}]. Error: "
					.$e->getMessage()
			);
		}
		return NULL;
	}

	private function prepareAttributes()
	{
		if (! is_null($this->ssl_attributes))
			return $this->ssl_attributes;

		if (0 >= (int)$this->packageID)
		{
			$this->ssl_attributes = array();
			return;
		}
		try
		{
			$this->ssl_attributes = DataObject::getMapOf(Factory()->Subscription($this->packageID)->PackageAttributes, "name", "value");

			$crypted_attrs = $this->cryptedUserParams();
			foreach ($this->ssl_attributes as $name => $value)
			{
				if (is_null($value)) {
					$this->ssl_attributes[$name] = $value;
					continue;
				}

				if (in_array($name, $crypted_attrs) && false !== ($value = $this->getCryptor()->decrypt($value)))
					$this->ssl_attributes[$name] = $value;
			}
		}
		catch (ProductException $e)
		{
			$this->valid = false;
			$this->addError(
				blmsg("TRANS_SSL_MODULE_ERROR"),
				"Cannot load attributes for ".get_class($this)."[{$this->pluginID}]. Error: "
					.$e->getMessage()
			);
		}
		return NULL;
	}

	private function getCryptor()
	{
		if (! is_null($this->cryptor))
		{
			$this->cryptor->errors = array();
			return $this->cryptor;
		}

		require_once mb_path("/lib-tk/include/crypt/crypt.php");
		return $this->cryptor = new Crypt(self::CRYPTOR_SECRET_KEY);
	}

	protected $valid = true;
	protected $capabilities;

	private $pluginID;
	private $productID;
	private $packageID;
	private $ssl_attributes;
	private $config;
	private $product;
	private $cryptor;
	private $done;
	private $countries;
	private $us_states;
}
